//
// *****************************************************************************
// |          G4LECS -- GEANT4 Low-Energy Compton Scattering package.          |
// |                          R. M. Kippen / LANL                              |
// |                                                                           |
// | G4LECSCompData.hh                                                         |
// |                                                                           |
// | Classes to read, store, and retrieve tabulated data for Compton           |
// | scattering, including Doppler broadening.                                 |
// |                                                                           |
// | RMK, DEC-2003. Change g4std headers to standard gcc for G4.6.0            |
// | RMK, MAY-2002. First version (based on G3LECS & G4CompositeEMDataSet.hh)  |
// *****************************************************************************
//
#ifndef G4LECSCOMPDATA_HH
#define G4LECSCOMPDATA_HH 1

#include "globals.hh"
#include "G4DataVector.hh"
#include <vector>

// ----------------------------------------------------------------------------|
// Class holds GLECS data for a single electron shell
// ----------------------------------------------------------------------------|
class G4LECSShellData
{ 
public:

  G4LECSShellData(){};

  ~G4LECSShellData(){};
 
public:

  G4DataVector cpCDF;        // Compton profile cumulative distribution

};


// ----------------------------------------------------------------------------|
// Class holds GLECS data for a single element
// ----------------------------------------------------------------------------|
class G4LECSElementData
{ 
public:

  G4LECSElementData(){};

  ~G4LECSElementData(){};
 
public:

  G4int nShell;                     // Number of electron shells for this Elem.
  G4DataVector shellOccupancy;      // Occupancy number for each shell
  G4DataVector shellSampler;        // Cumulative sampling factor for each shell
  G4DataVector shellPotential;      // Ionization potential (MeV) for each shell
  G4double meanPotential;           // Mean ionization potential (MeV) for all
  G4int nPoints;                    // Number of Compton profile grid points
  G4DataVector pProfile;            // Momentum at each grid point (AU)
  std::vector<G4LECSShellData> shellDat;  // Profile at each grid point

};

// ----------------------------------------------------------------------------|
// Class holds GLECS data for all elements
// ----------------------------------------------------------------------------|
class G4LECSCompData
{ 
public:

  G4LECSCompData(G4int minZ = 1, G4int maxZ = 99);

  G4LECSCompData(const G4String& dataFile,
		       G4int minZ = 1, G4int maxZ = 99);

  ~G4LECSCompData();
 
  G4int RandomAtomicShell(G4int Z) const;

  G4double RandomProfileMomentum(G4int Z, G4int iShell) const;

  G4double MaxTabulatedMomentum(G4int Z) const;

  G4double MeanIonizationEnergy(G4int Z) const;

  G4double ShellIonizationEnergy(G4int Z, G4int iShell) const;

  G4double LogLogIntrp(G4double xval, 
		       const G4DataVector& xdat, 
		       const G4DataVector& ydat) const;

  G4double TableInteg(G4double xlim1, G4double xlim2, 
		       const G4DataVector& xdat, 
		       const G4DataVector& ydat) const;

  void PrintData() const;

private:

  void LoadData(const G4String& fileName);

  std::vector<G4LECSElementData*> components;
  size_t nComponents;

  G4int zMin;
  G4int zMax;

};
 
#endif
