//
// *****************************************************************************
// |          G4LECS -- GEANT4 Low-Energy Compton Scattering package.          |
// |                          R. M. Kippen / LANL                              |
// |                                                                           |
// | G4LECSCompton.hh                                                          |
// |                                                                           |
// | Classes to initialize and perform the Compton (incoherent) scattering     |
// | process, including shellwise Doppler broadening.  Algorithms based on     |
// | those of Namito, Ban & Hirayama, NIM A349, 489-494 (1994).                |
// |                                                                           |
// | *** Present code for unpolarized radiation only ***                       |
// |                                                                           |
// | RMK, MAY-2002. First version (based on G3LECS & G4LowEnergyCompton.hh)    |
// *****************************************************************************
//
//
#ifndef G4LECSCOMPTON_HH
#define G4LECSCOMPTON_HH 1

#include "globals.hh"
#include "G4VDiscreteProcess.hh"

class G4Track;
class G4Step;
class G4ParticleDefinition;
class G4VParticleChange;
class G4VEMDataSet;
class G4LECSCompData;
class G4VCrossSectionHandler;
class G4VRangeTest;

class G4LECSCompton : public G4VDiscreteProcess {

public:
  
  G4LECSCompton(const G4String& processName ="G4LECSCompton");
  
  ~G4LECSCompton();

  G4bool IsApplicable(const G4ParticleDefinition& definition);
  
  void BuildPhysicsTable(const G4ParticleDefinition& photon);
 
  void PrintInfoDefinition();

  G4VParticleChange* PostStepDoIt(const G4Track& aTrack, const G4Step& aStep);
 
  // For testing purpose only
  G4double DumpMeanFreePath(const G4Track& aTrack, 
			    G4double previousStepSize, 
			    G4ForceCondition* condition) 
  { return GetMeanFreePath(aTrack, previousStepSize, condition); }

protected:

  G4double GetMeanFreePath(const G4Track& aTrack, 
			   G4double previousStepSize, 
			   G4ForceCondition* condition);

private: 

  // Hide copy constructor and assignment operator as private 
  G4LECSCompton& operator=(const G4LECSCompton& right);
  G4LECSCompton(const G4LECSCompton& );

  G4double lowEnergyLimit;  // low energy limit  applied to the process
  G4double highEnergyLimit; // high energy limit applied to the process

  G4VEMDataSet* meanFreePathTable;
  G4VEMDataSet* scatterFunctionData;
  G4LECSCompData* profileData;

  G4VCrossSectionHandler* crossSectionHandler;

  G4VRangeTest* rangeTest;

  const G4double intrinsicLowEnergyLimit; // intrinsic validity range
  const G4double intrinsicHighEnergyLimit;

};

#endif

